# Johnson solids

The Johnson solids are 92 different 3d-polytopes with regular faces that are not Platonic or Archimedean solids and neither belong to an infinite family of prisms or antiprisms (see [Wolfram Mathworld](https://mathworld.wolfram.com/JohnsonSolid.html)). This dataset includes their exact vertex and facet descriptions as well as approximate vertex and facet descriptions using floating point numbers.
Further data included consists of the incidence relations between vertices and facets, and a description of the field extension over the rational numbers that is needed to represent the data exactly.

## History
In 1966 Norman W. Johnson [[1]](#1) introduced the Johnson solids and conjectured that there were only 92 such polytopes.
In 1969 V.A. Zalgaller classified them and thereby proved that there are only 92 of them [[2]](#2). 
In 2009 A.V. Timofeenko found vertex coordinates for the noncomposite Johnson solids that do not lie in the quadratic extension of the ﬁeld of rational numbers [[3]](#3). 
Johnson solids have been implemented in `polymake` Version 3.0, January 18th, 2016 with floating point coordinates by Olivia Röhrig. 
[`OSCAR`](https://github.com/oscar-system/Oscar.jl) version `1.0` comes with the first known exact implementation of all Johnson solids, to the best of our knowledge.

This collection comprises serialized data for the geometry of all Johnson solids including exact coordinates in a suitable algebraic number field.
It employs the MaRDI file format [[4]](#4), which seeks to be FAIR, and which provides a mechanism for upgrading to future file formats.

Partial information about the Johnson solids can also be found elsewhere: 

At https://netlib.sandia.gov/polyhedra/ there is a database of polyhedra which also includes the Johnson solids. These solids are correct from a combinatorial viewpoint, but the vertices are expressed using floating point numbers only.

The site https://polytope.miraheze.org/wiki/Johnson_solid exhibits vertex coordinates symbolically, but there are errors.
As an example, the elongated pentagonal orthocupolarotunda (J40) there has `sqrt((5 + sqrt(5))/40)` as a `y`-coordinate where `sqrt((5 - sqrt(5))/40` would be correct.

## Installation

The data was created using `OSCAR` serialization (see [the documentation](https://docs.oscar-system.org/stable/General/serialization/)) and can be loaded in an active `OSCAR` session. This allows the user to easily ask for many additional properties, e.g. the volume of a given solid, as well as to perform algebraic precise computations within the fields over which the polytopes are described. Here is an example application:

```
julia> using Oscar
  ___   ____   ____    _    ____
 / _ \ / ___| / ___|  / \  |  _ \   |  Combining ANTIC, GAP, Polymake, Singular
| | | |\___ \| |     / _ \ | |_) |  |  Type "?Oscar" for more information
| |_| | ___) | |___ / ___ \|  _ <   |  Manual: https://docs.oscar-system.org
 \___/ |____/ \____/_/   \_\_| \_\  |  Version 1.1.0-DEV

julia> j = load("/your/path/to/j9")
Dict{String, Any} with 2 entries:
  "float"   => Polytope in ambient dimension 3 with Float64 type coefficients
  "precise" => Polytope in ambient dimension 3 with EmbeddedNumFieldElem{Hecke.…

julia> volume(j["float"])
2.021980232726787

julia> volume(j["precise"])
(5//8*sqrt(5) + 5//8)*a + 1//24*sqrt(5) + 5//24 (2.02)
```

While loading and processing this data is recommended with `OSCAR,` due to the format being `JSON` it can also be accessed by other means. We provide a sample script (`johnson_load.jl`) that only requires [`julia`](https://julialang.org/) and its [`JSON`](https://github.com/JuliaIO/JSON.jl) package.


## Usage

Include the `johnson-load.jl` file that comes with this collection by running `include("/your/path/to/johnson-load.jl")`. This gives access to the `load_johnson(i; format = :VERTICES)` function:

Parameters are
- `i::Int` The number of the corresponding johnson solid
- `format::Symbol` Can have one of six values

`format = :VERTICES` (the default)
Returns a `Matrix{String}`, each of whose 3 entry lines corresponds to a vertex, in exact coordinates, some of which may use a field extension of the rational numbers and thus include symbols corresponding to non-rational roots of rational polynomials. Their corresponding definition can be asked for using the `:FIELD` parameter.

`format = :FACETS`
Returns a `Matrix{String}`, each of whose 4 entry lines
["-d", "a", "b", "c"]
corresponds to a facet defining linear inequality
a*x1 + b*x2 + c*x3 <= d.

`format = :VERTICES_FLOAT`
Returns a `Matrix{Float64}`, each of whose 3 entry lines corresponds to approximate float coordinates of a vertex.

`format = :FACETS_FLOAT`
Returns a `Matrix{Any}`, each of whose 4 entry lines corresponds to approximate float coordinates of a facet defining inequality as above.

`format = :VERTICES_IN_FACETS`
Returns a `Vector{Vector{Int64}}` each of whose entries represents a facet and lists the indices of the vertices it contains. The numbering is compatible with the ordering of the vertices and facets listed above.

`format = :FIELD`
Returns 3-element `Vector{Any}` or the String "QQField". In the latter case there is no need for extending the field.
In the first case the the Vector has the form:
 [strings of names of the roots]
 [strings of the defining polynomials for these roots]
 String corresponding to the approximate value of the last defined root, that corresponds to the first entry each of the previous arrays.
In the case of a non-simple extension, an entry becomes an array over strings instead of just a string.

### Example

```
julia> load_johnson(13)
7×3 Matrix{String}:
 "(((1//2)))"                     "(((-1//4) + (-1//4)*sqrt(5))*a)"  "0"
 "(((-1//2)))"                    "(((-1//4) + (-1//4)*sqrt(5))*a)"  "0"
 "(((1//4) + (1//4)*sqrt(5)))"    "(((-1//4) + (1//4)*sqrt(5))*a)"   "0"
 "(((-1//4) + (-1//4)*sqrt(5)))"  "(((-1//4) + (1//4)*sqrt(5))*a)"   "0"
 "0"                              "(((1))*a)"                        "0"
 "0"                              "0"                                "(((-1//2) + (1//2)*sqrt(5))*a)"
 "0"                              "0"                                "(((1//2) + (-1//2)*sqrt(5))*a)"

julia> load_johnson(13; format=:FACETS)
10×4 Matrix{String}:
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "(((5//2) + (-3//2)*sqrt(5))*a)"  "(((-2) + (1)*sqrt(5)))"        "(((-1)))"
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "(((-5) + (2)*sqrt(5))*a)"        "(((1//2) + (-1//2)*sqrt(5)))"  "(((-1)))"
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "0"                               "(((3) + (-1)*sqrt(5)))"        "(((-1)))"
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "(((-5//2) + (3//2)*sqrt(5))*a)"  "(((-2) + (1)*sqrt(5)))"        "(((-1)))"
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "(((5) + (-2)*sqrt(5))*a)"        "(((1//2) + (-1//2)*sqrt(5)))"  "(((-1)))"
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "(((5//2) + (-3//2)*sqrt(5))*a)"  "(((-2) + (1)*sqrt(5)))"        "(((1)))"
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "(((-5) + (2)*sqrt(5))*a)"        "(((1//2) + (-1//2)*sqrt(5)))"  "(((1)))"
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "0"                               "(((3) + (-1)*sqrt(5)))"        "(((1)))"
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "(((-5//2) + (3//2)*sqrt(5))*a)"  "(((-2) + (1)*sqrt(5)))"        "(((1)))"
 "(((-1//2) + (1//2)*sqrt(5))*a)"  "(((5) + (-2)*sqrt(5))*a)"        "(((1//2) + (-1//2)*sqrt(5)))"  "(((1)))"

julia> load_johnson(13; format = :VERTICES_FLOAT)
7×3 Matrix{Float64}:
  0.5       -0.688191   0.0
 -0.5       -0.688191   0.0
  0.809017   0.262866   0.0
 -0.809017   0.262866   0.0
  0.0        0.850651   0.0
  0.0        0.0        0.525731
  0.0        0.0       -0.525731

julia> load_johnson(13; format = :FACETS_FLOAT)
10×4 Matrix{Float64}:
 0.525731  -0.726543   0.236068  -1.0
 0.525731  -0.449028  -0.618034  -1.0
 0.525731   0.0        0.763932  -1.0
 0.525731   0.726543   0.236068  -1.0
 0.525731   0.449028  -0.618034  -1.0
 0.525731  -0.726543   0.236068   1.0
 0.525731  -0.449028  -0.618034   1.0
 0.525731   0.0        0.763932   1.0
 0.525731   0.726543   0.236068   1.0
 0.525731   0.449028  -0.618034   1.0

julia> load_johnson(13; format=:VERTICES_IN_FACETS)
10-element Vector{Vector{Int64}}:
 [1, 3, 6]
 [3, 5, 6]
 [1, 2, 6]
 [2, 4, 6]
 [4, 5, 6]
 [1, 3, 7]
 [3, 5, 7]
 [1, 2, 7]
 [2, 4, 7]
 [4, 5, 7]

julia> load_johnson(13; format=:FIELD)
3-element Vector{Any}:
 ["a", "sqrt(5)"]
 ["((-1//2) + (-1//10)*sqrt(5)) + ((1))*x^2", "(-5) + (1)*x^2"]
 "0.850650808351929299533367156982421875 +/- 5.05850981423277244741765201752059510909020900726318359375e-13"
```

From the output of the last command we can e.g. read that `a` is a root of the polynomial `x^2 - (1//2 + sqrt(5)//10)`, and its embedded value is approximately `0.85`.

## References

<a id="1">[1]</a> 
Johnson, N. W. (1966).
Convex polyhedra with regular faces.
Canadian Journal of mathematics, 18, 169 - 200

<a id="2">[2]</a>
Zalgaller, V. A. (1969).
Convex polyhedra with regular faces.

<a id="3">[3]</a>
Timofeenko, A. V. (2009).
The non-platonic and non-Archimedean noncomposite polyhedra.
Journal of Mathematical Sciences, 162, 710-729.

<a id="4">[4]</a>
Della Vecchia, A., Joswig, M., Lorenz, B. (2023).
A FAIR File Format for Mathematical Software.
ArXiv preprint [2309.00465](https://arxiv.org/abs/2309.00465).
