_vector_to_string_monomial(v::Vector{Any}; rn::Vector = []) = _johnson_term(v, "x"; rn = rn)

function _vector_to_string_polynomial(v::Vector{Any}; rn::Vector = [])
    return join(_vector_to_string_monomial.(v; rn = rn), " + ")
end

function load_johnson(i::Int; format::Symbol = :VERTICES)
    data = read(joinpath(@__DIR__, string("j", i)), String)
    toplevel = JSON.parse(data)
    d = toplevel["data"]
    
    if format == :VERTICES_FLOAT
        return _johnson_vertices_float(d["float"]["VERTICES"])
    elseif format == :FACETS_FLOAT
        return _johnson_facets_float(d["float"]["FACETS"])
    elseif format == :VERTICES_IN_FACETS
        if haskey(toplevel, "_refs")
            return _johnson_incidence(d["precise"]["data"]["VERTICES_IN_FACETS"]["data"])
        else
            return _johnson_incidence(d["precise"]["VERTICES_IN_FACETS"])
        end
    end

    if !haskey(toplevel, "_refs")
        # for QQ
        format == :FIELD && return "QQField"
        root_names = []
        x = d["precise"]
    else
        # for number fields
        r = toplevel["_refs"]
        field_ref = toplevel["_type"]["params"]["precise"]["params"]
        ef = r[field_ref]["data"]

        nf = r[ef["num_field"]]["data"]

        root_names = _johnson_read_field(r, ef["num_field"])

        if format == :FIELD

            def_poly_strings = _johnson_read_field_information(r, ef["num_field"]; rn = root_names)
        
            if haskey(nf, "def_pol") # simple number field
                e = split(r[ef["embedding"]]["data"]["data"]["data"][1], " ")
                # e contains 4 hexadecimal values:
                # midpoint mantissa, midpoint exponent, radius mantissa, radius exponent (base = 2)
                e_int = parse.(BigInt, e, base = 16)
                e_strs = string(e_int[1]*2.0^e_int[2], " +/- ", e_int[3]*2.0^e_int[4])
            elseif haskey(nf, "def_pols") # non-simple number field
                e = broadcast(x -> split(x[1], " "), r[ef["embedding"]]["data"]["data"]["data"])
                e_ints = broadcast(x -> parse.(BigInt, x, base = 16), e)
                e_strs = [string(e_int[1]*2.0^e_int[2], " +/- ", e_int[3]*2.0^e_int[4]) for e_int in e_ints]
            end

            return [root_names, def_poly_strings, e_strs]
            
        end

        x = d["precise"]["data"]
    end
    
    if format == :VERTICES
        return _johnson_vertices(x["VERTICES"]; rn = root_names)
    end
    if format == :FACETS
        return _johnson_facets(x["FACETS"]; rn = root_names)
    end

end

function _johnson_read_field(r::Dict, s::String)
    nf = r[s]["data"]
    
    if haskey(nf, "def_pol") # simple number field
        dp = nf["def_pol"]["data"]
        root_names = [nf["var"]]
        b = r[nf["def_pol"]["_type"]["params"]]["data"]["base_ring"]
    elseif haskey(nf, "def_pols") # non-simple number field
        dps = nf["def_pols"]["data"]
        root_names = [nf["vars"]]
        b = r[nf["def_pols"]["_type"]["params"]["params"]]["data"]["base_ring"]
    end
    b isa Dict && b["_type"] == "QQField" && return root_names
    return vcat(root_names, _johnson_read_field(r, b))
end

function _johnson_read_field_information(r::Dict, s::String; rn::Vector = [])
    nf = r[s]["data"]
    
    if haskey(nf, "def_pol") # simple number field
        dp = nf["def_pol"]["data"]
        def_poly_strings = [_vector_to_string_polynomial(dp; rn = rn[begin+1:end])]
        b = r[nf["def_pol"]["_type"]["params"]]["data"]["base_ring"]
    elseif haskey(nf, "def_pols") # non-simple number field
        dps = nf["def_pols"]["data"]
        def_poly_strings = [_vector_to_string_polynomial.(dps; rn = rn[begin+1:end])]
        b = r[nf["def_pols"]["_type"]["params"]["params"]]["data"]["base_ring"]
    end
    b isa Dict && b["_type"] == "QQField" && return def_poly_strings
    return vcat(def_poly_strings, _johnson_read_field_information(r, b; rn = rn[begin+1:end]))
end

_johnson_coefficient(s::String; rn::Vector = []) = string("(", s, ")")

function _johnson_term(b::Vector{Any}, s::String; rn = [])
    res = _johnson_coefficient(b[2]; rn = rn)
    if b[1] != "0"
        res = string(res, "*", s)
        if b[1] != "1"
            res = string(res, "^", b[1])
        end
    end
    return res
end

function _johnson_variables(b::Vector{Any}, v::Vector)
    return [b[1][i] != "1" ? string(v[i], "^", b[1][i]) : v[i] for i in 1:length(v) if b[1][i] != "0"]
end

function _johnson_term(b::Vector{Any}, v::Vector; rn = [])
    jv = _johnson_variables(b, v)
    if length(jv)!=0
        return string(_johnson_coefficient(b[2]; rn = rn), "*", join(jv, "*"))
    end
    return _johnson_coefficient(b[2]; rn = rn)
end

function _johnson_coefficient(a::Vector{Any}; rn::Vector = [])
    length(a) == 0 && return "0"
    return string("(", join([_johnson_term(b, rn[1]; rn = rn[begin+1:end]) for b in a], " + "), ")")
end

function _johnson_facets(v::Vector{Any}; rn::Vector = [])
    res = vcat([permutedims(w) for w in v]...)
    return _johnson_coefficient.(res; rn = rn)
end

_johnson_vertices(v::Vector{Any}; rn::Vector = []) = _johnson_facets(v; rn = rn)[:, begin+1:end]

function _johnson_facets_float(v::Vector{Any})
    return convert.(Float64, vcat([permutedims(w) for w in v]...))
end

_johnson_vertices_float(v::Vector{Any}; rn::Vector = []) = _johnson_facets_float(v)[:, begin+1:end]

function _johnson_incidence(v::Vector{Any})
    return [x .+ 1 for x in v[1:end-1]]
end
